#include <vector>
#include "doctest/extensions/doctest_mpi.h"
#include "pdm.h"
#include "pdm_doctest.h"
#include "pdm_part_mesh_nodal_elmts_utils.h"
#include "pdm_generate_mesh.h"
#include "pdm_priv.h"
#include "pdm_logging.h"
#include "pdm_error.h"
#include "pdm_array.h"

static PDM_part_mesh_nodal_t *
_generate_mesh
(
 PDM_MPI_Comm         comm,
 PDM_Mesh_nodal_elt_t elt_type
 )
{
  int              n_part      = 1;
  PDM_g_num_t      n_vtx_seg   = 4;
  PDM_split_dual_t part_method = PDM_SPLIT_DUAL_WITH_HILBERT;

  int mesh_dimension = PDM_Mesh_nodal_elt_dim_get(elt_type);

  if (mesh_dimension == 2) {
    return PDM_generate_mesh_rectangle(comm,
                                       elt_type,
                                       1,
                                       NULL,
                                       0.,
                                       0.,
                                       0.,
                                       1.,
                                       1.,
                                       n_vtx_seg,
                                       n_vtx_seg,
                                       n_part,
                                       part_method);
  }
  else {
    return PDM_generate_mesh_parallelepiped(comm,
                                            elt_type,
                                            1,
                                            NULL,
                                            0.,
                                            0.,
                                            0.,
                                            1.,
                                            1.,
                                            1.,
                                            n_vtx_seg,
                                            n_vtx_seg,
                                            n_vtx_seg,
                                            n_part,
                                            part_method);
  }
}


MPI_TEST_CASE("[pdm_part_mesh_nodal_elmts_utils] - part_mesh_nodal_elmts_compute_child_entities", 1) {

  PDM_MPI_Comm pdm_comm = PDM_MPI_mpi_2_pdm_mpi_comm(&test_comm);

  PDM_Mesh_nodal_elt_t elt_type             = PDM_MESH_NODAL_N_ELEMENT_TYPES;
  PDM_bool_t           compute_parent_child = PDM_TRUE;

  int expected_n_entity[PDM_GEOMETRY_KIND_MAX];
  PDM_array_reset_int(expected_n_entity, PDM_GEOMETRY_KIND_MAX, 0);

  std::vector<int> expected_child_to_parent_idx [PDM_GEOMETRY_KIND_MAX];
  std::vector<int> expected_child_to_parent     [PDM_GEOMETRY_KIND_MAX];
  std::vector<int> expected_entity_to_vtx_idx   [PDM_GEOMETRY_KIND_MAX];
  std::vector<int> expected_entity_to_vtx       [PDM_GEOMETRY_KIND_MAX];
  std::vector<int> expected_parent_to_entity_idx[PDM_GEOMETRY_KIND_MAX];
  std::vector<int> expected_parent_to_entity    [PDM_GEOMETRY_KIND_MAX];

  SUBCASE("TRIA3") {
    elt_type = PDM_MESH_NODAL_TRIA3;
    expected_child_to_parent_idx[PDM_GEOMETRY_KIND_RIDGE] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12};
    expected_child_to_parent    [PDM_GEOMETRY_KIND_RIDGE] = {1, 3, 5, 14, 16, 18, 1, 7, 13, 6, 12, 18};
    SUBCASE("TRIA3, compute_parent_child = 1") {
      printf("TRIA3, compute_parent_child = 1\n");
      compute_parent_child = PDM_TRUE;
      expected_n_entity[PDM_GEOMETRY_KIND_RIDGE] = 33;
      expected_entity_to_vtx_idx   [PDM_GEOMETRY_KIND_RIDGE] = {0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56, 58, 60, 62, 64, 66};
      expected_entity_to_vtx       [PDM_GEOMETRY_KIND_RIDGE] = {1, 2, 2, 3, 3, 4, 14, 13, 15, 14, 16, 15, 5, 1, 9, 5, 13, 9, 4, 8, 8, 12, 12, 16, 2, 5, 2, 6, 6, 3, 7, 3, 7, 4, 5, 6, 7, 6, 8, 7, 6, 9, 10, 6, 7, 10, 11, 7, 11, 8, 9, 10, 11, 10, 12, 11, 13, 10, 10, 14, 11, 14, 15, 11, 15, 12};
      expected_parent_to_entity_idx[PDM_GEOMETRY_KIND_RIDGE] = {0, 3, 6, 9, 12, 15, 18, 21, 24, 27, 30, 33, 36, 39, 42, 45, 48, 51, 54};
      expected_parent_to_entity    [PDM_GEOMETRY_KIND_RIDGE] = {1, 7, 13, -13, 14, -18, 2, -14, -15, 15, -16, 19, 3, 16, -17, 10, 17, 20, 8, 18, 21, -21, -22, -26, -19, 22, 23, -23, -24, 27, -20, 24, -25, 11, 25, 28, 9, 26, -29, 4, 29, 30, -27, -30, 31, 5, -31, -32, -28, 32, -33, 6, 12, 33};

    };
    SUBCASE("TRIA3, compute_parent_child = 0") {
      printf("TRIA3, compute_parent_child = 0\n");
      compute_parent_child = PDM_FALSE;
    };
  }

  SUBCASE("QUAD4") {
    elt_type = PDM_MESH_NODAL_QUAD4;
    expected_child_to_parent_idx[PDM_GEOMETRY_KIND_RIDGE] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12};
    expected_child_to_parent    [PDM_GEOMETRY_KIND_RIDGE] = {1, 2, 3, 7, 8, 9, 1, 4, 7, 3, 6, 9};
    SUBCASE("QUAD4, compute_parent_child = 1") {
      printf("QUAD4, compute_parent_child = 1\n");
      compute_parent_child = PDM_TRUE;
      expected_n_entity[PDM_GEOMETRY_KIND_RIDGE] = 24;
      expected_entity_to_vtx_idx   [PDM_GEOMETRY_KIND_RIDGE] = {0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48};
      expected_entity_to_vtx       [PDM_GEOMETRY_KIND_RIDGE] = {1, 2, 2, 3, 3, 4, 14, 13, 15, 14, 16, 15, 5, 1, 9, 5, 13, 9, 4, 8, 8, 12, 12, 16, 2, 6, 3, 7, 6, 5, 7, 6, 8, 7, 10, 6, 11, 7, 10, 9, 11, 10, 11, 12, 10, 14, 15, 11};
      expected_parent_to_entity_idx[PDM_GEOMETRY_KIND_RIDGE] = {0, 4, 8, 12, 16, 20, 24, 28, 32, 36};
      expected_parent_to_entity    [PDM_GEOMETRY_KIND_RIDGE] = {1, 7, 13, 15, 2, -13, 14, 16, 3, 10, -14, 17, 8, -15, -18, 20, -16, 18, -19, 21, 11, -17, 19, -22, 4, 9, -20, 23, 5, -21, -23, -24, 6, 12, 22, 24};
    };
    SUBCASE("QUAD4, compute_parent_child = 0") {
      printf("QUAD4, compute_parent_child = 0\n");
      compute_parent_child = PDM_FALSE;
    };
  }

  SUBCASE("POLY_2D") {
    elt_type = PDM_MESH_NODAL_POLY_2D;
    expected_child_to_parent_idx[PDM_GEOMETRY_KIND_RIDGE] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28};
    expected_child_to_parent    [PDM_GEOMETRY_KIND_RIDGE] = {9, 17, 1, 18, 2, 19, 10, 11, 23, 3, 24, 4, 25, 12, 9, 17, 5, 20, 6, 23, 11, 10, 19, 7, 22, 8, 25, 12};
    SUBCASE("POLY_2D, compute_parent_child = 1") {
      printf("POLY_2D, compute_parent_child = 1\n");
      compute_parent_child = PDM_TRUE;
      expected_n_entity[PDM_GEOMETRY_KIND_RIDGE] = 76;
      expected_entity_to_vtx_idx   [PDM_GEOMETRY_KIND_RIDGE] = {0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56, 58, 60, 62, 64, 66, 68, 70, 72, 74, 76, 78, 80, 82, 84, 86, 88, 90, 92, 94, 96, 98, 100, 102, 104, 106, 108, 110, 112, 114, 116, 118, 120, 122, 124, 126, 128, 130, 132, 134, 136, 138, 140, 142, 144, 146, 148, 150, 152};
      expected_entity_to_vtx       [PDM_GEOMETRY_KIND_RIDGE] = {49, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 50, 43, 51, 44, 43, 45, 44, 46, 45, 47, 46, 48, 47, 52, 48, 7, 49, 11, 7, 21, 11, 25, 21, 35, 25, 39, 35, 51, 39, 50, 10, 10, 14, 14, 24, 24, 28, 28, 38, 38, 42, 42, 52, 7, 1, 8, 2, 3, 8, 4, 9, 9, 5, 10, 6, 12, 8, 13, 9, 15, 11, 12, 16, 12, 17, 15, 16, 13, 18, 19, 13, 14, 20, 17, 18, 21, 15, 16, 22, 20, 19, 17, 22, 18, 23, 23, 19, 24, 20, 26, 22, 27, 23, 29, 25, 26, 30, 31, 26, 30, 29, 27, 32, 27, 33, 28, 34, 32, 31, 29, 35, 36, 30, 36, 31, 33, 34, 37, 32, 37, 33, 38, 34, 40, 36, 37, 41, 39, 43, 40, 44, 40, 45, 41, 46, 47, 41, 48, 42};
      expected_parent_to_entity_idx[PDM_GEOMETRY_KIND_RIDGE] = {0, 3, 6, 9, 12, 15, 18, 21, 24, 27, 30, 33, 36, 40, 44, 48, 52, 60, 68, 76, 84, 92, 100, 108, 116, 124};
      expected_parent_to_entity    [PDM_GEOMETRY_KIND_RIDGE] = {3, 30, 31, 5, -32, -33, 10, -72, 73, 12, -74, -75, 17, -37, -45, 19, -54, 62, 24, -43, 51, 26, -60, 68, 1, 15, -29, 7, 22, 34, 8, 21, 71, 14, 28, 76, -38, 39, -46, 48, -41, -42, -49, -50, -55, -56, 63, -64, -58, 59, 66, -67, 2, 16, 29, -30, -35, 37, 38, -40, 4, -31, 32, 35, -36, -39, 41, -44, 6, 23, 33, -34, 36, 42, 43, 47, 18, 40, 45, 46, -52, 54, 55, 57, 44, -48, 49, 52, -53, 56, 58, 61, 25, -47, 50, -51, 53, -59, 60, -65, 9, 20, -57, -62, -63, -69, -71, 72, 11, -61, 64, -66, 69, 70, -73, 74, 13, 27, 65, 67, -68, -70, 75, -76};
    };
    SUBCASE("POLY_2D, compute_parent_child = 0") {
      printf("POLY_2D, compute_parent_child = 0\n");
      compute_parent_child = PDM_FALSE;
    };
  }

  SUBCASE("TETRA4") {
    elt_type = PDM_MESH_NODAL_TETRA4;
    expected_child_to_parent_idx[PDM_GEOMETRY_KIND_SURFACIC] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107, 108};
    expected_child_to_parent    [PDM_GEOMETRY_KIND_SURFACIC] = {1, 4, 6, 8, 11, 14, 16, 18, 21, 24, 26, 28, 31, 34, 36, 38, 41, 44, 92, 93, 99, 97, 102, 103, 109, 107, 112, 113, 119, 117, 122, 123, 129, 127, 132, 133, 1, 3, 18, 19, 31, 33, 48, 49, 61, 63, 78, 79, 91, 93, 108, 109, 121, 123, 14, 12, 26, 27, 44, 42, 56, 57, 74, 72, 86, 87, 104, 102, 116, 117, 134, 132, 1, 2, 49, 46, 91, 92, 9, 6, 51, 52, 99, 96, 11, 12, 59, 56, 101, 102, 33, 34, 78, 77, 123, 124, 38, 37, 83, 84, 128, 127, 43, 44, 88, 87, 133, 134};
    expected_child_to_parent_idx[PDM_GEOMETRY_KIND_RIDGE   ] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36};
    expected_child_to_parent    [PDM_GEOMETRY_KIND_RIDGE   ] = {1, 6, 11, 34, 38, 44, 1, 18, 31, 14, 26, 44, 92, 99, 102, 123, 127, 133, 93, 109, 123, 102, 117, 132, 1, 49, 91, 12, 56, 102, 33, 78, 123, 44, 87, 134};
    SUBCASE("TETRA4, compute_parent_child = 1") {
      printf("TETRA4, compute_parent_child = 1\n");
      compute_parent_child = PDM_TRUE;
      expected_n_entity[PDM_GEOMETRY_KIND_SURFACIC] = 324;
      expected_n_entity[PDM_GEOMETRY_KIND_RIDGE   ] = 252;
      expected_entity_to_vtx_idx   [PDM_GEOMETRY_KIND_SURFACIC] = {0, 3, 6, 9, 12, 15, 18, 21, 24, 27, 30, 33, 36, 39, 42, 45, 48, 51, 54, 57, 60, 63, 66, 69, 72, 75, 78, 81, 84, 87, 90, 93, 96, 99, 102, 105, 108, 111, 114, 117, 120, 123, 126, 129, 132, 135, 138, 141, 144, 147, 150, 153, 156, 159, 162, 165, 168, 171, 174, 177, 180, 183, 186, 189, 192, 195, 198, 201, 204, 207, 210, 213, 216, 219, 222, 225, 228, 231, 234, 237, 240, 243, 246, 249, 252, 255, 258, 261, 264, 267, 270, 273, 276, 279, 282, 285, 288, 291, 294, 297, 300, 303, 306, 309, 312, 315, 318, 321, 324, 327, 330, 333, 336, 339, 342, 345, 348, 351, 354, 357, 360, 363, 366, 369, 372, 375, 378, 381, 384, 387, 390, 393, 396, 399, 402, 405, 408, 411, 414, 417, 420, 423, 426, 429, 432, 435, 438, 441, 444, 447, 450, 453, 456, 459, 462, 465, 468, 471, 474, 477, 480, 483, 486, 489, 492, 495, 498, 501, 504, 507, 510, 513, 516, 519, 522, 525, 528, 531, 534, 537, 540, 543, 546, 549, 552, 555, 558, 561, 564, 567, 570, 573, 576, 579, 582, 585, 588, 591, 594, 597, 600, 603, 606, 609, 612, 615, 618, 621, 624, 627, 630, 633, 636, 639, 642, 645, 648, 651, 654, 657, 660, 663, 666, 669, 672, 675, 678, 681, 684, 687, 690, 693, 696, 699, 702, 705, 708, 711, 714, 717, 720, 723, 726, 729, 732, 735, 738, 741, 744, 747, 750, 753, 756, 759, 762, 765, 768, 771, 774, 777, 780, 783, 786, 789, 792, 795, 798, 801, 804, 807, 810, 813, 816, 819, 822, 825, 828, 831, 834, 837, 840, 843, 846, 849, 852, 855, 858, 861, 864, 867, 870, 873, 876, 879, 882, 885, 888, 891, 894, 897, 900, 903, 906, 909, 912, 915, 918, 921, 924, 927, 930, 933, 936, 939, 942, 945, 948, 951, 954, 957, 960, 963, 966, 969, 972};
      expected_entity_to_vtx       [PDM_GEOMETRY_KIND_SURFACIC] = {1, 5, 2, 6, 2, 5, 3, 2, 7, 6, 7, 2, 3, 7, 4, 8, 4, 7, 6, 5, 10, 9, 10, 5, 6, 10, 7, 11, 7, 10, 8, 7, 12, 11, 12, 7, 9, 13, 10, 14, 10, 13, 11, 10, 15, 14, 15, 10, 11, 15, 12, 16, 12, 15, 50, 54, 49, 53, 49, 54, 50, 51, 54, 55, 54, 51, 52, 56, 51, 55, 51, 56, 53, 54, 57, 58, 57, 54, 55, 59, 54, 58, 54, 59, 55, 56, 59, 60, 59, 56, 58, 62, 57, 61, 57, 62, 58, 59, 62, 63, 62, 59, 60, 64, 59, 63, 59, 64, 1, 17, 5, 21, 5, 17, 9, 5, 25, 21, 25, 5, 9, 25, 13, 29, 13, 25, 21, 17, 37, 33, 37, 17, 21, 37, 25, 41, 25, 37, 29, 25, 45, 41, 45, 25, 33, 49, 37, 53, 37, 49, 41, 37, 57, 53, 57, 37, 41, 57, 45, 61, 45, 57, 8, 24, 4, 20, 4, 24, 8, 12, 24, 28, 24, 12, 16, 32, 12, 28, 12, 32, 20, 24, 36, 40, 36, 24, 28, 44, 24, 40, 24, 44, 28, 32, 44, 48, 44, 32, 40, 56, 36, 52, 36, 56, 40, 44, 56, 60, 56, 44, 48, 64, 44, 60, 44, 64, 1, 2, 17, 18, 17, 2, 33, 17, 34, 18, 34, 17, 33, 34, 49, 50, 49, 34, 18, 2, 19, 3, 19, 2, 18, 19, 34, 35, 34, 19, 50, 34, 51, 35, 51, 34, 3, 4, 19, 20, 19, 4, 35, 19, 36, 20, 36, 19, 35, 36, 51, 52, 51, 36, 29, 30, 13, 14, 13, 30, 29, 45, 30, 46, 30, 45, 61, 62, 45, 46, 45, 62, 14, 30, 15, 31, 15, 30, 46, 47, 30, 31, 30, 47, 46, 62, 47, 63, 47, 62, 31, 32, 15, 16, 15, 32, 31, 47, 32, 48, 32, 47, 63, 64, 47, 48, 47, 64, 2, 5, 17, 2, 7, 19, 2, 5, 22, 3, 19, 7, 6, 22, 2, 4, 7, 19, 2, 7, 22, 6, 5, 22, 7, 4, 24, 6, 7, 22, 5, 10, 22, 6, 10, 22, 8, 7, 24, 7, 22, 10, 5, 10, 25, 2, 22, 17, 2, 18, 22, 2, 19, 22, 7, 12, 24, 22, 5, 17, 7, 10, 27, 9, 25, 10, 11, 27, 7, 7, 12, 27, 4, 24, 19, 10, 11, 27, 10, 13, 25, 7, 19, 22, 5, 22, 21, 23, 7, 19, 11, 12, 27, 24, 19, 7, 5, 25, 22, 10, 27, 15, 23, 22, 7, 10, 13, 30, 11, 15, 27, 7, 24, 23, 14, 10, 30, 12, 27, 15, 10, 30, 15, 22, 27, 7, 7, 23, 27, 18, 22, 17, 22, 10, 25, 26, 10, 22, 7, 24, 27, 15, 12, 32, 27, 10, 22, 19, 22, 18, 21, 17, 22, 10, 25, 26, 20, 24, 19, 10, 27, 26, 12, 24, 27, 23, 19, 22, 10, 30, 25, 23, 24, 19, 26, 10, 30, 10, 30, 27, 28, 27, 12, 21, 25, 22, 30, 13, 25, 27, 32, 12, 23, 27, 22, 15, 27, 30, 17, 34, 22, 26, 22, 25, 15, 31, 27, 23, 27, 24, 18, 34, 22, 32, 15, 27, 26, 27, 22, 19, 22, 34, 17, 37, 22, 28, 27, 24, 19, 24, 36, 21, 37, 22, 19, 22, 39, 26, 30, 25, 23, 39, 19, 19, 39, 24, 27, 30, 26, 29, 25, 30, 22, 23, 39, 22, 37, 25, 23, 39, 24, 28, 32, 27, 22, 39, 27, 17, 34, 37, 31, 27, 30, 25, 22, 42, 23, 27, 39, 26, 42, 22, 24, 39, 27, 31, 32, 27, 22, 27, 42, 19, 39, 34, 26, 25, 42, 35, 19, 39, 22, 34, 37, 38, 22, 34, 19, 36, 39, 27, 24, 44, 26, 42, 27, 39, 22, 34, 25, 42, 30, 22, 37, 38, 26, 42, 30, 36, 24, 39, 28, 27, 44, 27, 30, 42, 38, 39, 22, 25, 30, 45, 22, 42, 37, 38, 22, 42, 22, 42, 39, 27, 32, 44, 24, 39, 40, 34, 37, 33, 27, 30, 47, 42, 37, 25, 31, 27, 47, 27, 32, 47, 24, 44, 39, 27, 39, 42, 34, 39, 35, 25, 42, 41, 27, 43, 39, 38, 34, 37, 35, 39, 36, 44, 39, 27, 38, 34, 39, 25, 42, 45, 27, 42, 43, 27, 44, 43, 40, 39, 36, 42, 47, 27, 37, 42, 38, 27, 43, 47, 42, 30, 45, 27, 47, 44, 30, 46, 42, 38, 42, 39, 47, 30, 42, 34, 49, 37, 41, 42, 37, 44, 32, 47, 39, 44, 40, 34, 39, 51, 43, 42, 39, 34, 37, 54, 35, 39, 51, 38, 54, 34, 36, 51, 39, 43, 39, 44, 34, 54, 39, 42, 45, 41, 38, 37, 54, 38, 39, 54, 39, 36, 56, 43, 47, 42, 46, 42, 45, 37, 54, 42, 38, 42, 54, 43, 47, 44, 46, 42, 47, 39, 42, 54, 40, 39, 56, 37, 42, 57, 49, 54, 34, 50, 34, 54, 48, 47, 44, 39, 56, 44, 34, 54, 51, 54, 49, 37, 41, 42, 57, 39, 42, 59, 43, 59, 39, 39, 59, 44, 51, 56, 36, 42, 45, 57, 43, 42, 59, 53, 54, 37, 51, 39, 54, 39, 55, 51, 56, 39, 51, 43, 59, 44, 55, 54, 39, 42, 59, 47, 37, 54, 57, 43, 59, 47, 45, 42, 62, 46, 42, 62, 55, 56, 39, 44, 59, 47, 42, 62, 47, 39, 59, 54, 39, 55, 59, 42, 54, 57, 39, 59, 56, 58, 42, 54, 59, 54, 42, 44, 47, 64, 58, 57, 42, 58, 59, 42, 44, 56, 59, 42, 62, 57, 58, 42, 62, 44, 59, 60, 42, 62, 59, 62, 45, 57, 44, 64, 59, 59, 47, 62, 47, 63, 59, 64, 59, 47};
      expected_parent_to_entity_idx[PDM_GEOMETRY_KIND_SURFACIC] = {0, 4, 8, 12, 16, 20, 24, 28, 32, 36, 40, 44, 48, 52, 56, 60, 64, 68, 72, 76, 80, 84, 88, 92, 96, 100, 104, 108, 112, 116, 120, 124, 128, 132, 136, 140, 144, 148, 152, 156, 160, 164, 168, 172, 176, 180, 184, 188, 192, 196, 200, 204, 208, 212, 216, 220, 224, 228, 232, 236, 240, 244, 248, 252, 256, 260, 264, 268, 272, 276, 280, 284, 288, 292, 296, 300, 304, 308, 312, 316, 320, 324, 328, 332, 336, 340, 344, 348, 352, 356, 360, 364, 368, 372, 376, 380, 384, 388, 392, 396, 400, 404, 408, 412, 416, 420, 424, 428, 432, 436, 440, 444, 448, 452, 456, 460, 464, 468, 472, 476, 480, 484, 488, 492, 496, 500, 504, 508, 512, 516, 520, 524, 528, 532, 536, 540};
      expected_parent_to_entity    [PDM_GEOMETRY_KIND_SURFACIC] = {1, 37, 73, 109, 74, -124, -125, 152, 38, -128, -137, 159, 2, -111, 113, 116, -109, 111, 124, 128, 3, 80, -110, -112, 136, -138, -143, -164, 4, -113, 115, -118, 79, 125, -126, 158, 110, -115, 126, -136, 5, 85, 112, 114, 56, 86, -133, 161, 138, 140, -146, -166, 6, 55, 117, 121, -114, -117, 133, -140, 7, -116, -119, 120, -153, -154, 160, -176, 8, 39, 123, 130, 40, 137, 141, -170, 119, -123, -141, 153, 9, 118, -120, -122, 143, 150, -151, 173, 154, -157, -162, -181, 10, -129, 131, -134, 122, 129, -150, 157, 11, 57, -121, -127, 58, 163, -169, 184, 12, -131, 132, -139, 146, 151, -155, -178, 127, -132, 155, -163, 13, 41, -130, 135, -160, -165, 167, 188, 42, 91, -171, 192, 14, 92, -144, -147, -135, 144, 165, 171, 15, 134, 142, 145, 98, 174, 177, -199, 16, 97, 147, -149, 162, -167, 168, 191, -142, 149, -168, -174, 17, 139, -145, -148, 60, 169, 172, 196, 103, -177, -180, -204, 18, 59, 104, 156, 148, -156, -172, 180, 76, -152, 175, -179, 209, -210, 216, -238, 43, -159, -183, 186, 44, 75, -198, 228, -175, 183, 198, -209, 81, -158, 179, 182, 82, -206, 208, -235, 210, -214, 221, 241, 164, -187, 189, -193, -182, 187, 206, 214, 61, 88, -161, -185, 62, -218, 227, 245, 166, -189, -190, 195, 87, -208, -211, -239, 185, 190, 211, 218, 45, 170, -186, -194, -216, -223, 224, -247, 46, 230, -236, -255, 176, 200, 202, 207, 194, -200, 223, -230, -173, 193, 197, 201, 234, 237, 243, 259, 181, -202, 205, 213, -221, -224, 225, -252, -197, -205, -225, -234, 178, -195, -201, -203, 64, -227, -233, -257, -237, 240, -244, 264, 63, -184, 212, 219, 203, -212, 233, -240, -188, -207, 215, -217, 94, -249, 251, -271, 47, 93, -192, 222, 48, 236, -242, 266, -215, -222, 242, 249, -191, -213, 217, 220, -243, 246, -248, 270, 99, -251, -253, 275, 100, 199, -229, -231, -220, 229, -246, 253, 65, -196, -219, -226, 66, 106, -256, 281, 105, 204, 231, 232, 244, 248, 250, -274, 226, -232, -250, 256, 49, 77, -228, -254, 19, 78, 279, 280, 20, 50, 284, 292, 238, -260, 262, 267, 254, 260, -279, -284, 84, 235, -258, 261, 22, -293, 294, -297, -241, -262, -265, -268, 21, 83, -280, 283, 258, 265, -283, 293, 89, 239, -261, -263, 23, 68, 90, 289, 24, -294, -295, 303, 67, -245, 269, 277, 263, -269, -289, 295, 247, -267, 272, 273, 26, 308, -310, -313, 51, 255, 278, -285, 25, 52, -292, -299, -272, -278, 299, -308, 252, 268, -273, 276, 27, 297, -306, -307, 28, 310, 311, 314, -259, -286, 287, 291, -276, 286, 306, -311, 69, 257, -277, 282, 30, 70, 315, 318, -264, -287, -288, 296, 29, -303, 307, 309, -282, 288, -309, -315, 53, -266, 285, 290, 31, 313, -316, 317, 32, 54, 95, -320, 96, 271, 301, -302, -290, -301, 316, 320, -270, -291, 298, -300, 34, 102, -322, 323, 101, -275, 302, -305, 33, -314, -317, 319, -298, 305, -319, 322, 274, -296, 300, -304, 35, 72, -318, -321, 36, 107, -323, 324, 71, 108, -281, -312, 304, 312, 321, -324};
      expected_entity_to_vtx_idx   [PDM_GEOMETRY_KIND_RIDGE   ] = {0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56, 58, 60, 62, 64, 66, 68, 70, 72, 74, 76, 78, 80, 82, 84, 86, 88, 90, 92, 94, 96, 98, 100, 102, 104, 106, 108, 110, 112, 114, 116, 118, 120, 122, 124, 126, 128, 130, 132, 134, 136, 138, 140, 142, 144, 146, 148, 150, 152, 154, 156, 158, 160, 162, 164, 166, 168, 170, 172, 174, 176, 178, 180, 182, 184, 186, 188, 190, 192, 194, 196, 198, 200, 202, 204, 206, 208, 210, 212, 214, 216, 218, 220, 222, 224, 226, 228, 230, 232, 234, 236, 238, 240, 242, 244, 246, 248, 250, 252, 254, 256, 258, 260, 262, 264, 266, 268, 270, 272, 274, 276, 278, 280, 282, 284, 286, 288, 290, 292, 294, 296, 298, 300, 302, 304, 306, 308, 310, 312, 314, 316, 318, 320, 322, 324, 326, 328, 330, 332, 334, 336, 338, 340, 342, 344, 346, 348, 350, 352, 354, 356, 358, 360, 362, 364, 366, 368, 370, 372, 374, 376, 378, 380, 382, 384, 386, 388, 390, 392, 394, 396, 398, 400, 402, 404, 406, 408, 410, 412, 414, 416, 418, 420, 422, 424, 426, 428, 430, 432, 434, 436, 438, 440, 442, 444, 446, 448, 450, 452, 454, 456, 458, 460, 462, 464, 466, 468, 470, 472, 474, 476, 478, 480, 482, 484, 486, 488, 490, 492, 494, 496, 498, 500, 502, 504};
      expected_entity_to_vtx       [PDM_GEOMETRY_KIND_RIDGE   ] = {1, 2, 2, 3, 3, 4, 14, 13, 15, 14, 16, 15, 5, 1, 9, 5, 13, 9, 4, 8, 8, 12, 12, 16, 49, 50, 50, 51, 51, 52, 61, 62, 62, 63, 63, 64, 49, 53, 53, 57, 57, 61, 52, 56, 56, 60, 60, 64, 1, 17, 17, 33, 33, 49, 4, 20, 20, 36, 36, 52, 13, 29, 29, 45, 45, 61, 16, 32, 32, 48, 48, 64, 5, 2, 6, 2, 7, 2, 7, 3, 6, 5, 7, 4, 6, 7, 10, 5, 8, 7, 10, 6, 10, 7, 7, 11, 9, 10, 12, 7, 2, 17, 2, 18, 2, 19, 10, 11, 5, 17, 3, 19, 4, 19, 12, 11, 13, 10, 2, 22, 10, 14, 15, 10, 7, 19, 21, 5, 15, 11, 5, 22, 12, 15, 24, 4, 6, 22, 7, 22, 5, 25, 23, 7, 24, 7, 8, 24, 22, 10, 7, 27, 9, 25, 10, 25, 17, 18, 12, 24, 26, 10, 10, 27, 18, 19, 25, 13, 11, 27, 17, 21, 12, 27, 22, 17, 20, 19, 28, 12, 22, 18, 10, 30, 22, 19, 15, 27, 19, 23, 19, 24, 21, 22, 30, 13, 24, 20, 14, 30, 12, 32, 23, 22, 30, 15, 25, 21, 15, 31, 32, 15, 25, 22, 23, 24, 22, 26, 27, 22, 27, 23, 17, 34, 27, 24, 26, 25, 28, 24, 18, 34, 19, 34, 26, 27, 17, 37, 29, 25, 35, 19, 30, 25, 27, 28, 19, 36, 26, 30, 34, 22, 27, 30, 19, 39, 21, 37, 31, 27, 37, 22, 29, 30, 27, 32, 36, 24, 28, 32, 22, 38, 22, 39, 31, 30, 23, 39, 25, 37, 24, 39, 32, 31, 24, 40, 22, 42, 39, 27, 41, 25, 33, 34, 25, 42, 24, 44, 26, 42, 42, 27, 34, 35, 43, 27, 25, 45, 37, 33, 37, 34, 36, 35, 27, 44, 42, 30, 28, 44, 38, 34, 34, 39, 27, 47, 35, 39, 38, 37, 36, 39, 30, 45, 36, 40, 32, 44, 30, 46, 39, 38, 30, 47, 37, 41, 31, 47, 42, 37, 32, 47, 39, 40, 42, 38, 42, 39, 39, 43, 42, 41, 44, 39, 34, 49, 50, 34, 44, 40, 34, 51, 43, 42, 37, 49, 45, 41, 35, 51, 36, 51, 45, 42, 43, 44, 46, 42, 34, 54, 42, 47, 39, 51, 53, 37, 47, 43, 37, 54, 46, 45, 44, 47, 36, 56, 38, 54, 48, 44, 39, 54, 46, 47, 37, 57, 39, 55, 39, 56, 48, 47, 42, 54, 40, 56, 39, 59, 41, 57, 57, 42, 58, 42, 56, 44, 42, 59, 43, 59, 45, 57, 44, 59, 54, 49, 42, 62, 54, 50, 60, 44, 54, 51, 55, 51, 47, 59, 45, 62, 51, 56, 53, 54, 44, 64, 46, 62, 62, 47, 55, 54, 47, 63, 57, 54, 56, 55, 64, 47, 54, 58, 59, 54, 59, 55, 56, 59, 58, 57, 59, 58, 62, 57, 60, 59, 62, 58, 62, 59, 59, 63, 64, 59};
      expected_parent_to_entity_idx[PDM_GEOMETRY_KIND_RIDGE   ] = {0, 6, 12, 18, 24, 30, 36, 42, 48, 54, 60, 66, 72, 78, 84, 90, 96, 102, 108, 114, 120, 126, 132, 138, 144, 150, 156, 162, 168, 174, 180, 186, 192, 198, 204, 210, 216, 222, 228, 234, 240, 246, 252, 258, 264, 270, 276, 282, 288, 294, 300, 306, 312, 318, 324, 330, 336, 342, 348, 354, 360, 366, 372, 378, 384, 390, 396, 402, 408, 414, 420, 426, 432, 438, 444, 450, 456, 462, 468, 474, 480, 486, 492, 498, 504, 510, 516, 522, 528, 534, 540, 546, 552, 558, 564, 570, 576, 582, 588, 594, 600, 606, 612, 618, 624, 630, 636, 642, 648, 654, 660, 666, 672, 678, 684, 690, 696, 702, 708, 714, 720, 726, 732, 738, 744, 750, 756, 762, 768, 774, 780, 786, 792, 798, 804, 810};
      expected_parent_to_entity    [PDM_GEOMETRY_KIND_RIDGE   ] = {1, 7, 25, -37, 51, 55, -51, -52, -60, -79, -88, 91, -55, 64, -66, 86, 88, 97, 37, -38, 41, 60, 66, 69, 37, 51, 55, 60, -66, 88, 2, 39, -40, 53, 56, 63, 63, -70, -72, 93, -95, 102, 38, -39, -43, 60, 69, 70, 52, 53, -60, -83, 91, -93, 39, 53, 60, -63, 70, -93, 3, 40, -42, 56, 57, 63, -28, -57, 68, 89, 96, 99, -63, 72, 73, 95, -96, 108, 10, 42, 45, -68, -73, 74, 42, 57, 63, -68, 73, -96, -41, 44, -46, 66, 69, -75, -75, -78, -81, 107, -109, 114, 8, -44, -49, 71, 77, 78, -64, 66, -71, -97, 104, -107, 44, 66, 71, 75, 78, -107, 43, 46, -47, 69, 70, -75, -70, 72, -76, 102, -110, 111, 75, 81, -82, 109, 110, 118, 47, 48, -54, 76, 82, 85, 47, 70, -75, 76, -82, 110, 11, -45, 50, -73, 74, 80, 80, -87, -90, 113, 115, -123, -48, -50, 58, 76, 85, 87, -72, -73, -76, -108, 111, -113, 50, -73, 76, -80, 87, -113, 9, 49, -59, 77, 78, -84, -78, 81, -92, 114, -122, -125, -31, 84, 98, -120, 122, 132, 4, 59, 61, 92, -98, 100, 59, 78, -84, 92, 98, 122, 54, 62, -65, 82, 85, 94, 94, 103, 105, -127, 130, 138, 5, -61, -62, 92, 100, -103, -81, 82, -92, -118, -125, 127, 62, 82, 92, -94, -103, 127, -58, 65, 67, 85, 87, 94, -87, 90, -101, -123, 133, -135, -94, -105, 106, -130, -133, -142, 6, 12, 34, -67, 101, -106, -67, 87, 94, 101, 106, -133, 79, 88, -91, 112, 116, -126, -126, 131, 136, 156, 161, 165, -86, -88, -97, 119, 129, -131, 26, 112, -119, -147, 155, -156, 88, 112, 119, 126, -131, -156, 83, 91, -93, 116, 117, -126, -117, 121, -128, -152, 162, -164, 126, -136, -137, -161, -162, -171, 93, 95, 102, 128, 137, 139, 93, 117, -126, 128, -137, -162, 29, -89, -96, -99, 124, -134, -134, -141, 143, -166, -168, -177, -95, 96, -108, 128, 139, 141, -121, 124, -128, 157, -164, 166, -96, 124, 128, 134, 141, 166, 97, 104, -107, 129, -131, 140, 131, -136, -144, 165, -175, 178, -140, 146, -148, 173, 175, -181, 107, 109, 114, 144, 148, 150, 107, -131, 140, 144, -148, 175, -102, 110, -111, 137, 139, -145, -145, 151, -153, 179, -180, 187, -109, -110, -118, 144, 150, -151, 136, 137, -144, 171, 178, -179, 110, 137, 144, 145, -151, -179, 108, 111, -113, 139, 141, -145, -141, -143, -149, -177, -182, 185, 145, 153, -158, 180, 182, 193, 113, -115, -123, 149, 158, 160, 113, 141, -145, 149, -158, 182, -114, 122, 125, 148, 150, -159, -159, -167, 170, 192, 194, 201, 32, 120, -122, -132, 154, 167, -146, 148, -154, 181, 189, -192, 122, 148, 154, 159, 167, -192, 118, -125, 127, 150, -151, -159, 151, 153, -163, 187, 196, 199, 159, -170, -172, -194, -196, 207, -127, -130, 138, 163, 172, 174, -127, -151, -159, 163, -172, -196, 123, -133, 135, 158, 160, 169, 35, 169, -176, -202, 205, 211, 130, 133, 142, 163, 174, 176, -153, 158, -163, -193, 199, 202, -133, 158, 163, -169, 176, 202, 27, 147, 155, -156, 183, 188, -13, -183, 184, -195, -223, 225, 19, -188, 198, -200, 223, 232, 156, -161, 165, 195, 200, 204, 156, 183, 188, 195, -200, 223, 152, -162, 164, 186, 190, 197, 197, -206, 209, 227, 228, 236, 161, 162, 171, 195, 204, 206, -14, -184, 186, -195, 225, -227, -162, 186, 195, -197, 206, -227, -157, -164, 166, 190, 191, 197, -15, -22, -30, -191, -203, 231, -197, -209, -210, -228, -231, -239, -166, 168, -177, 203, 210, 213, -166, 191, 197, 203, -210, -231, -165, 175, -178, 200, 204, 212, 212, 216, -217, 238, -241, 245, -173, -175, 181, 208, 215, -216, -20, -198, 200, -208, -232, -238, 175, 200, 208, -212, -216, -238, -171, 178, -179, 204, 206, 212, -206, -209, -214, 236, -242, 243, -212, 217, -219, 241, 242, -246, 179, 180, 187, 214, 219, 220, 179, 206, 212, 214, -219, 242, 177, 182, -185, 210, 213, -218, -23, -218, -222, -226, -244, 248, -180, -182, -193, 214, 220, 222, 209, 210, -214, 239, 243, 244, 182, 210, 214, 218, 222, 244, -181, 189, -192, 215, -216, 221, 216, 217, -224, 245, -247, 249, 16, 21, -33, -221, -230, 247, 192, -194, 201, 224, 230, 234, 192, -216, 221, 224, -230, 247, -187, -196, -199, 219, 220, 229, -17, 229, 235, 237, 250, -251, 194, 196, -207, 224, 234, -235, -217, 219, -224, 246, 249, -250, -196, 219, 224, -229, -235, -250, 193, 199, 202, 220, 222, 229, -24, -222, 226, -233, 248, -252, 18, -229, -237, 240, 251, 252, 36, -202, -205, 211, 233, -240, -202, 222, 229, 233, 240, 252};
    };
    SUBCASE("TETRA4, compute_parent_child = 0") {
      printf("TETRA4, compute_parent_child = 0\n");
      compute_parent_child = PDM_FALSE;
    };
  }

  SUBCASE("PYRAMID5") {
    elt_type = PDM_MESH_NODAL_PYRAMID5;
    expected_child_to_parent_idx[PDM_GEOMETRY_KIND_SURFACIC] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81};
    expected_child_to_parent    [PDM_GEOMETRY_KIND_SURFACIC] = {2, 3, 4, 6, 8, 9, 10, 11, 14, 13, 16, 17, 20, 21, 22, 24, 26, 27, 3, 1, 10, 12, 21, 19, 29, 30, 39, 38, 47, 48, 57, 55, 64, 66, 75, 73, 1, 2, 29, 28, 55, 56, 4, 5, 32, 31, 58, 59, 7, 8, 35, 34, 61, 62, 55, 59, 61, 66, 69, 72, 73, 77, 79, 8, 17, 26, 34, 43, 52, 62, 71, 80, 21, 48, 75, 24, 51, 78, 27, 54, 81};
    expected_child_to_parent_idx[PDM_GEOMETRY_KIND_RIDGE   ] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 17, 19, 21, 22, 23, 24, 26, 28, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 41, 43, 45};
    expected_child_to_parent    [PDM_GEOMETRY_KIND_RIDGE   ] = {2, 4, 8, 21, 24, 27, 3, 10, 21, 8, 17, 26, 55, 59, 61, 73, 75, 78, 77, 81, 79, 55, 66, 73, 61, 62, 72, 71, 79, 80, 1, 29, 55, 8, 34, 62, 21, 48, 75, 27, 26, 54, 52, 80, 81};
    SUBCASE("PYRAMID5, compute_parent_child = 1") {
      printf("PYRAMID5, compute_parent_child = 1\n");
      compute_parent_child = PDM_TRUE;
      expected_n_entity[PDM_GEOMETRY_KIND_SURFACIC] = 243;
      expected_n_entity[PDM_GEOMETRY_KIND_RIDGE   ] = 225;
      expected_entity_to_vtx_idx   [PDM_GEOMETRY_KIND_SURFACIC] = {0, 3, 6, 9, 12, 15, 18, 21, 24, 27, 30, 33, 36, 39, 42, 45, 48, 51, 54, 57, 60, 63, 66, 69, 72, 75, 78, 81, 84, 87, 90, 93, 96, 99, 102, 105, 108, 111, 114, 117, 120, 123, 126, 129, 132, 135, 138, 141, 144, 147, 150, 153, 156, 159, 162, 166, 170, 174, 178, 182, 186, 190, 194, 198, 202, 206, 210, 214, 218, 222, 226, 230, 234, 238, 242, 246, 250, 254, 258, 262, 266, 270, 273, 276, 279, 282, 285, 288, 291, 294, 297, 300, 303, 306, 309, 312, 315, 318, 322, 325, 328, 331, 334, 338, 341, 344, 347, 350, 353, 357, 360, 363, 366, 369, 373, 376, 379, 383, 386, 389, 392, 395, 398, 401, 404, 407, 410, 413, 416, 419, 423, 426, 430, 433, 437, 440, 443, 447, 450, 453, 456, 459, 462, 465, 468, 472, 475, 478, 481, 484, 487, 491, 494, 497, 500, 503, 506, 509, 513, 516, 519, 522, 525, 528, 531, 534, 537, 540, 543, 547, 550, 553, 556, 560, 564, 567, 570, 573, 576, 580, 584, 587, 590, 593, 596, 600, 603, 606, 609, 612, 616, 619, 622, 626, 629, 632, 635, 638, 641, 644, 647, 650, 653, 656, 659, 662, 665, 668, 671, 674, 677, 680, 683, 687, 690, 693, 696, 699, 702, 705, 708, 711, 714, 717, 720, 723, 726, 729, 732, 735, 738, 741, 744, 747, 750, 753, 756, 759, 763, 767, 771, 775, 779, 783};
      expected_entity_to_vtx       [PDM_GEOMETRY_KIND_SURFACIC] = {2, 1, 6, 5, 6, 1, 2, 6, 3, 7, 3, 6, 4, 3, 8, 7, 8, 3, 5, 9, 6, 10, 6, 9, 7, 6, 11, 10, 11, 6, 7, 11, 8, 12, 8, 11, 10, 9, 14, 13, 14, 9, 10, 14, 11, 15, 11, 14, 12, 11, 16, 15, 16, 11, 5, 1, 21, 17, 21, 1, 5, 21, 9, 25, 9, 21, 13, 9, 29, 25, 29, 9, 17, 33, 21, 37, 21, 33, 25, 21, 41, 37, 41, 21, 25, 41, 29, 45, 29, 41, 37, 33, 53, 49, 53, 33, 37, 53, 41, 57, 41, 53, 45, 41, 61, 57, 61, 41, 17, 1, 18, 2, 18, 1, 17, 18, 33, 34, 33, 18, 49, 33, 50, 34, 50, 33, 2, 3, 18, 19, 18, 3, 34, 18, 35, 19, 35, 18, 34, 35, 50, 51, 50, 35, 19, 3, 20, 4, 20, 3, 19, 20, 35, 36, 35, 20, 51, 35, 52, 36, 52, 35, 49, 50, 54, 53, 50, 51, 55, 54, 51, 52, 56, 55, 53, 54, 58, 57, 54, 55, 59, 58, 55, 56, 60, 59, 57, 58, 62, 61, 58, 59, 63, 62, 59, 60, 64, 63, 4, 8, 24, 20, 8, 12, 28, 24, 12, 16, 32, 28, 20, 24, 40, 36, 24, 28, 44, 40, 28, 32, 48, 44, 36, 40, 56, 52, 40, 44, 60, 56, 44, 48, 64, 60, 13, 29, 30, 14, 29, 45, 46, 30, 45, 61, 62, 46, 14, 30, 31, 15, 30, 46, 47, 31, 46, 62, 63, 47, 15, 31, 32, 16, 31, 47, 48, 32, 47, 63, 64, 48, 1, 22, 6, 3, 6, 22, 3, 23, 7, 3, 8, 24, 9, 22, 6, 11, 22, 6, 11, 7, 23, 1, 22, 18, 11, 24, 8, 3, 18, 22, 1, 22, 21, 9, 10, 26, 3, 23, 19, 11, 10, 26, 3, 24, 20, 3, 23, 22, 18, 22, 6, 2, 3, 23, 24, 11, 12, 28, 9, 22, 21, 9, 14, 30, 6, 22, 21, 5, 11, 14, 30, 11, 23, 22, 11, 15, 31, 9, 26, 22, 11, 24, 23, 6, 22, 23, 7, 11, 26, 22, 11, 16, 32, 9, 25, 26, 11, 23, 27, 23, 24, 8, 7, 11, 28, 24, 11, 27, 26, 26, 22, 6, 10, 9, 26, 30, 11, 27, 28, 11, 26, 30, 9, 30, 29, 11, 27, 31, 11, 32, 28, 11, 30, 31, 33, 18, 22, 11, 32, 31, 35, 18, 22, 33, 22, 21, 35, 23, 19, 21, 22, 18, 17, 35, 24, 20, 26, 30, 14, 10, 35, 22, 23, 18, 22, 23, 19, 35, 23, 24, 41, 22, 21, 23, 24, 20, 19, 43, 22, 23, 41, 26, 22, 43, 23, 24, 43, 22, 26, 41, 26, 25, 33, 22, 38, 43, 23, 27, 26, 22, 21, 25, 35, 22, 38, 43, 28, 24, 43, 26, 27, 35, 39, 23, 41, 26, 30, 23, 22, 26, 27, 43, 27, 28, 35, 24, 40, 43, 30, 26, 41, 29, 30, 41, 38, 22, 43, 27, 31, 28, 24, 23, 27, 43, 28, 32, 43, 38, 22, 43, 31, 30, 43, 23, 39, 33, 38, 34, 43, 31, 32, 35, 38, 34, 43, 40, 24, 33, 38, 37, 41, 42, 26, 26, 30, 29, 25, 35, 36, 40, 43, 42, 26, 35, 39, 38, 18, 22, 38, 34, 31, 30, 26, 27, 35, 39, 40, 43, 28, 44, 41, 37, 38, 41, 30, 46, 28, 32, 31, 27, 38, 22, 21, 37, 43, 46, 30, 43, 38, 39, 41, 42, 38, 43, 47, 31, 23, 22, 38, 39, 43, 40, 39, 43, 48, 32, 43, 42, 38, 33, 54, 38, 40, 24, 23, 39, 35, 38, 54, 43, 40, 44, 38, 22, 26, 42, 41, 42, 46, 35, 55, 39, 35, 56, 40, 43, 42, 46, 41, 46, 45, 41, 54, 38, 43, 38, 54, 43, 44, 48, 43, 47, 46, 33, 54, 50, 43, 55, 39, 43, 47, 48, 35, 54, 50, 43, 40, 56, 33, 53, 54, 35, 55, 51, 41, 58, 42, 43, 42, 58, 35, 56, 52, 46, 30, 26, 42, 35, 54, 55, 35, 55, 56, 43, 44, 60, 41, 53, 54, 41, 46, 62, 43, 46, 62, 43, 54, 55, 43, 63, 47, 41, 58, 54, 43, 56, 55, 43, 54, 58, 43, 48, 64, 41, 58, 57, 43, 55, 59, 43, 60, 56, 43, 59, 58, 41, 62, 58, 43, 60, 59, 43, 62, 58, 41, 62, 61, 43, 63, 59, 43, 60, 64, 43, 62, 63, 43, 64, 63, 50, 54, 38, 34, 53, 54, 38, 37, 38, 54, 55, 39, 40, 56, 55, 39, 58, 54, 38, 42, 46, 62, 58, 42};
      expected_parent_to_entity_idx[PDM_GEOMETRY_KIND_SURFACIC] = {0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 70, 75, 80, 85, 90, 95, 100, 105, 110, 115, 120, 125, 130, 135, 140, 145, 150, 155, 160, 165, 170, 175, 180, 185, 190, 195, 200, 205, 210, 215, 220, 225, 230, 235, 240, 245, 250, 255, 260, 265, 270, 275, 280, 285, 290, 295, 300, 305, 310, 315, 320, 325, 330, 335, 340, 345, 350, 355, 360, 365, 370, 375, 380, 385, 390, 395, 400, 405};
      expected_parent_to_entity    [PDM_GEOMETRY_KIND_SURFACIC] = {20, 37, 89, -92, -130, 1, 38, 82, -89, -98, 2, 19, -82, 92, -103, 3, 43, 83, -91, 98, 44, 91, 94, -97, -134, 4, -83, -84, 97, 109, 49, -94, 96, 99, -137, 5, 50, 64, -85, -96, 6, 84, 85, -99, 114, 7, 21, 86, -101, 103, 8, -86, -93, -107, 117, 22, 101, 107, 112, -145, 10, 87, 95, 110, -117, 9, -87, -88, -105, -109, 105, -110, -113, -116, -151, 11, 88, 90, -108, -114, 12, 65, -90, -100, -115, 108, 113, 115, 119, -158, 24, -112, -118, -121, 169, 13, 93, -102, 118, -132, 14, 23, 73, 102, 121, 15, -95, 104, -120, 132, 116, 120, -122, 124, 174, 16, 76, -104, 106, -124, -119, 122, 123, -126, 179, 17, 66, 100, -111, -123, 18, 79, -106, 111, 126, 40, -125, -143, -163, 173, 25, 39, 125, 128, 130, 26, -128, 143, 167, 180, 46, -127, -129, -133, 134, 45, 127, 146, 165, -173, 133, -146, -149, 172, 185, 52, 67, 131, -153, 170, 51, 129, -131, -135, 137, 135, 149, 153, -175, 190, 139, -156, 168, -183, 193, 28, 136, 156, 177, -180, 27, -136, -139, 142, 145, -138, -160, -162, 182, -185, 141, 160, -171, 188, -193, 138, -141, 144, -148, 151, 68, 147, -166, -176, 192, -140, 162, 166, -186, -190, 140, -144, -147, -152, 158, -150, -168, -178, 194, -213, 29, -142, 150, -155, -169, 30, 74, 155, 178, 198, 148, 154, 157, 161, -174, -154, 171, -181, -197, 213, 77, -161, 181, -184, 202, 69, -159, 176, 187, 201, 152, -157, 159, -164, -179, 80, 164, 184, -187, -205, 32, 41, 55, 203, 208, 42, 163, 189, -203, -238, 31, -167, -189, -208, 239, 47, -165, 191, 206, 238, 48, 56, -206, 209, 214, -172, -191, -195, -214, 240, 53, 57, -209, 212, 215, 54, 70, -170, 196, -212, 175, 195, -196, -215, -241, 33, -177, 199, 217, -239, 183, -199, 210, -222, 242, 34, 58, -217, 222, -226, -188, -200, 211, -224, -242, -182, 200, 204, 220, -240, 59, -220, 224, -227, -229, 186, -204, -207, -223, 241, 71, -192, 207, -216, -228, 60, 223, 227, 228, -231, 36, 61, 226, 230, -233, -194, -210, -218, -230, 243, 35, 75, -198, 218, 233, 197, -211, 219, 232, -243, 62, 229, -232, 234, 236, 78, -202, -219, -221, -236, 63, 231, -234, -235, -237, 72, -201, 216, -225, 235, 81, 205, 221, 225, 237};
      expected_entity_to_vtx_idx   [PDM_GEOMETRY_KIND_RIDGE   ] = {0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56, 58, 60, 62, 64, 66, 68, 70, 72, 74, 76, 78, 80, 82, 84, 86, 88, 90, 92, 94, 96, 98, 100, 102, 104, 106, 108, 110, 112, 114, 116, 118, 120, 122, 124, 126, 128, 130, 132, 134, 136, 138, 140, 142, 144, 146, 148, 150, 152, 154, 156, 158, 160, 162, 164, 166, 168, 170, 172, 174, 176, 178, 180, 182, 184, 186, 188, 190, 192, 194, 196, 198, 200, 202, 204, 206, 208, 210, 212, 214, 216, 218, 220, 222, 224, 226, 228, 230, 232, 234, 236, 238, 240, 242, 244, 246, 248, 250, 252, 254, 256, 258, 260, 262, 264, 266, 268, 270, 272, 274, 276, 278, 280, 282, 284, 286, 288, 290, 292, 294, 296, 298, 300, 302, 304, 306, 308, 310, 312, 314, 316, 318, 320, 322, 324, 326, 328, 330, 332, 334, 336, 338, 340, 342, 344, 346, 348, 350, 352, 354, 356, 358, 360, 362, 364, 366, 368, 370, 372, 374, 376, 378, 380, 382, 384, 386, 388, 390, 392, 394, 396, 398, 400, 402, 404, 406, 408, 410, 412, 414, 416, 418, 420, 422, 424, 426, 428, 430, 432, 434, 436, 438, 440, 442, 444, 446, 448, 450};
      expected_entity_to_vtx       [PDM_GEOMETRY_KIND_RIDGE   ] = {1, 2, 2, 3, 3, 4, 14, 13, 15, 14, 16, 15, 5, 1, 9, 5, 13, 9, 4, 8, 8, 12, 12, 16, 49, 50, 50, 51, 51, 52, 61, 62, 62, 63, 63, 64, 49, 53, 53, 57, 57, 61, 52, 56, 56, 60, 60, 64, 1, 17, 17, 33, 33, 49, 4, 20, 20, 36, 36, 52, 13, 29, 29, 45, 45, 61, 16, 32, 32, 48, 48, 64, 6, 1, 2, 6, 6, 3, 7, 3, 8, 3, 5, 6, 6, 7, 8, 7, 6, 9, 10, 6, 6, 11, 7, 11, 18, 1, 8, 11, 10, 9, 2, 18, 18, 3, 10, 11, 21, 1, 19, 3, 20, 3, 12, 11, 14, 9, 22, 1, 10, 14, 22, 3, 14, 11, 23, 3, 5, 21, 15, 11, 16, 11, 24, 3, 6, 22, 7, 23, 21, 9, 22, 9, 8, 24, 22, 11, 23, 11, 25, 9, 18, 17, 26, 9, 24, 11, 10, 26, 18, 19, 26, 11, 29, 9, 27, 11, 21, 17, 28, 11, 20, 19, 30, 9, 22, 18, 28, 12, 30, 11, 31, 11, 19, 23, 21, 22, 32, 11, 30, 14, 20, 24, 22, 23, 21, 25, 15, 31, 24, 23, 22, 26, 27, 23, 25, 26, 18, 33, 34, 18, 24, 28, 26, 27, 18, 35, 21, 33, 19, 35, 29, 25, 22, 33, 28, 27, 20, 35, 30, 26, 22, 35, 23, 35, 31, 27, 37, 21, 29, 30, 24, 35, 28, 32, 38, 22, 31, 30, 39, 23, 21, 41, 31, 32, 22, 41, 40, 24, 22, 43, 23, 43, 25, 41, 26, 41, 34, 33, 24, 43, 42, 26, 34, 35, 26, 43, 27, 43, 37, 33, 29, 41, 30, 41, 36, 35, 28, 43, 38, 33, 28, 44, 34, 38, 30, 43, 38, 35, 39, 35, 31, 43, 40, 35, 32, 43, 37, 38, 46, 30, 40, 36, 38, 39, 31, 47, 37, 41, 38, 41, 39, 40, 42, 38, 38, 43, 39, 43, 40, 43, 42, 41, 50, 33, 40, 44, 34, 50, 42, 43, 50, 35, 51, 35, 53, 33, 45, 41, 44, 43, 46, 41, 52, 35, 54, 33, 42, 46, 46, 43, 54, 35, 47, 43, 55, 35, 37, 53, 45, 46, 48, 43, 56, 35, 38, 54, 48, 44, 47, 46, 39, 55, 53, 41, 54, 41, 47, 48, 40, 56, 54, 43, 57, 41, 55, 43, 58, 41, 56, 43, 58, 42, 58, 43, 61, 41, 59, 43, 62, 41, 60, 43, 44, 60, 50, 54, 62, 43, 63, 43, 51, 55, 64, 43, 53, 54, 62, 46, 55, 54, 63, 47, 56, 55, 54, 58, 59, 55, 58, 57, 58, 59, 59, 60, 58, 62, 63, 59};
      expected_parent_to_entity_idx[PDM_GEOMETRY_KIND_RIDGE   ] = {0, 8, 16, 24, 32, 40, 48, 56, 64, 72, 80, 88, 96, 104, 112, 120, 128, 136, 144, 152, 160, 168, 176, 184, 192, 200, 208, 216, 224, 232, 240, 248, 256, 264, 272, 280, 288, 296, 304, 312, 320, 328, 336, 344, 352, 360, 368, 376, 384, 392, 400, 408, 416, 424, 432, 440, 448, 456, 464, 472, 480, 488, 496, 504, 512, 520, 528, 536, 544, 552, 560, 568, 576, 584, 592, 600, 608, 616, 624, 632, 640, 648};
      expected_parent_to_entity    [PDM_GEOMETRY_KIND_RIDGE   ] = {-25, 49, 55, 60, 77, -85, 89, 94, -1, 37, -38, 49, 52, 60, -69, -89, 7, 37, 42, 55, 60, -65, 69, -94, 2, 38, 39, -52, 53, 62, 69, 89, 53, 56, 62, 64, -81, -89, -93, 98, 39, 40, 43, 62, 64, -69, 70, -98, 56, 57, 64, 68, 87, 93, -97, -101, -3, -10, 28, 41, 57, 68, -73, 97, 40, 41, -44, 64, 68, -70, 73, 101, -8, -42, 45, 65, -69, 71, 72, 94, 45, 46, 51, 69, 72, 78, -80, 102, 71, 72, 76, 78, -94, 99, -102, 104, -46, 47, 54, -69, 74, 80, 82, -102, -43, 47, 48, 69, -70, 74, 75, 98, 74, 75, 82, 84, -98, 102, 103, 108, 44, 48, 50, 70, -73, 75, 79, -101, -11, 50, 58, 73, 79, 86, 90, 107, 75, 79, 84, 86, 101, -103, -107, -114, 76, 78, 83, 88, -104, -112, 116, 121, 51, 59, -61, 78, 80, 88, 96, -116, -4, 9, -31, 59, 83, 88, -96, -121, 54, 61, 63, -80, 82, 91, -96, 116, 82, 84, 91, 92, -108, -116, 119, -125, -5, 63, 66, 91, 92, 96, 100, 125, 84, 86, 92, 95, 114, -119, -123, 128, -12, -34, 58, 67, 86, -90, 95, 123, -6, 34, 66, 67, 92, 95, -100, -128, 89, 105, -106, 113, 124, 135, 146, 148, 26, -77, 85, -89, -94, 105, 110, 113, 94, 110, 113, 120, -124, 141, 146, -155, 81, 89, 93, -98, 109, 111, 117, 118, -89, 106, 109, 117, -124, 138, -148, 150, 98, 117, 118, 124, -126, 150, 151, -158, 29, -97, 115, 122, 130, 144, 153, -157, -87, -93, 97, 101, 111, 115, 118, 122, -101, 118, 122, 126, -130, 151, 153, -162, -102, -124, 129, 134, 137, 161, -163, 167, -94, -120, 124, 127, 129, 155, 160, 161, 94, -99, 102, -104, 127, 129, 133, 134, -98, -124, 126, 131, 132, 158, 164, 165, 102, 124, 131, -137, 139, 163, 164, 171, 98, -102, -103, -108, 131, 132, 139, 140, -107, -130, 136, 145, -147, 166, 169, 176, 101, -126, 130, 132, 136, 162, 165, 166, -101, 103, 107, 114, 132, 136, 140, 145, 116, 134, -137, 143, 156, 167, 177, 180, 104, 112, -116, -121, 133, 134, 142, 143, -32, 121, 142, 143, -156, 175, 177, -186, 108, 116, -119, 125, 139, 140, 149, 152, -116, 137, 139, 149, -156, 171, -180, 181, -125, 149, 152, 156, 159, 181, 183, 191, -35, -123, 145, 147, 154, 176, 187, -190, -114, 119, 123, -128, 140, 145, 152, 154, 35, 128, 152, 154, -159, 183, 187, -195, -13, 19, -27, 168, 174, 179, -209, 214, 135, 146, -148, 168, 170, 179, -189, 209, 141, 146, 155, 174, 179, -185, 189, -214, 138, 148, 150, -170, 172, 182, 189, -209, -14, 172, 173, 182, 184, 209, -212, -216, 150, 151, 158, 182, 184, -189, 192, 216, -15, -22, 173, 178, 184, 188, 212, -218, 22, 30, 144, 153, 157, 178, 188, -196, 151, 153, 162, 184, 188, -192, 196, 218, -155, 160, 161, 185, -189, 193, 194, 214, 161, 163, 167, 189, 194, 200, 202, 219, 20, 193, 194, 198, 200, -214, -219, -221, -163, 164, 171, -189, 197, -202, 203, -219, -158, 164, 165, 189, -192, 197, 199, -216, 197, 199, 203, 205, 216, 219, 220, 222, -162, 165, 166, 192, -196, 199, 201, -218, 23, 166, -169, 176, 196, 201, 207, -208, -23, 199, 201, 205, 207, 218, -220, 223, 16, 21, 198, 200, 204, 206, 221, -224, 167, 177, -180, 200, -202, 206, 215, 224, -16, -33, 175, 177, 186, 204, 206, -215, 171, 180, 181, 202, 203, 210, -215, -224, 17, 203, 205, 210, 211, -222, 224, 225, -17, 181, 183, -191, 210, 211, 215, -217, 18, -24, 205, 207, 211, 213, -223, -225, 24, -36, 176, 187, 190, 207, 208, 213, -18, 36, 183, 187, 195, 211, 213, 217};
    };
    SUBCASE("PYRAMID5, compute_parent_child = 0") {
      printf("PYRAMID5, compute_parent_child = 0\n");
      compute_parent_child = PDM_FALSE;
    };
  }

  SUBCASE("PRISM6") {
    elt_type = PDM_MESH_NODAL_PRISM6;
    expected_child_to_parent_idx[PDM_GEOMETRY_KIND_SURFACIC] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72};
    expected_child_to_parent    [PDM_GEOMETRY_KIND_SURFACIC] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 1, 7, 13, 19, 25, 31, 37, 43, 49, 6, 12, 18, 24, 30, 36, 42, 48, 54, 1, 19, 37, 3, 21, 39, 5, 23, 41, 14, 32, 50, 16, 34, 52, 18, 36, 54};
    expected_child_to_parent_idx[PDM_GEOMETRY_KIND_RIDGE   ] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 29, 31, 33, 35, 37, 39, 40, 41, 42};
    expected_child_to_parent    [PDM_GEOMETRY_KIND_RIDGE   ] = {1, 3, 5, 14, 16, 18, 1, 7, 13, 6, 12, 18, 37, 39, 41, 50, 52, 54, 37, 43, 49, 42, 48, 54, 1, 19, 37, 5, 6, 24, 23, 42, 41, 14, 13, 31, 32, 50, 49, 18, 36, 54};
    SUBCASE("PRISM6, compute_parent_child = 1") {
      printf("PRISM6, compute_parent_child = 1\n");
      compute_parent_child = PDM_TRUE;
      expected_n_entity[PDM_GEOMETRY_KIND_SURFACIC] = 171;
      expected_n_entity[PDM_GEOMETRY_KIND_RIDGE   ] = 180;
      expected_entity_to_vtx_idx   [PDM_GEOMETRY_KIND_SURFACIC] = {0, 3, 6, 9, 12, 15, 18, 21, 24, 27, 30, 33, 36, 39, 42, 45, 48, 51, 54, 57, 60, 63, 66, 69, 72, 75, 78, 81, 84, 87, 90, 93, 96, 99, 102, 105, 108, 112, 116, 120, 124, 128, 132, 136, 140, 144, 148, 152, 156, 160, 164, 168, 172, 176, 180, 184, 188, 192, 196, 200, 204, 208, 212, 216, 220, 224, 228, 232, 236, 240, 244, 248, 252, 256, 260, 264, 268, 272, 276, 279, 283, 286, 289, 293, 296, 300, 304, 307, 311, 314, 317, 321, 325, 329, 332, 335, 338, 342, 345, 349, 353, 356, 359, 363, 367, 370, 374, 377, 380, 384, 387, 390, 393, 396, 399, 403, 406, 409, 413, 417, 420, 424, 427, 431, 435, 438, 441, 444, 448, 451, 455, 459, 462, 465, 469, 473, 476, 480, 483, 487, 490, 494, 497, 501, 504, 508, 512, 516, 520, 524, 528, 532, 536, 540, 544, 548, 552, 556, 560, 564, 568, 572, 576, 580, 584, 588, 592, 596, 600, 604, 608, 612};
      expected_entity_to_vtx       [PDM_GEOMETRY_KIND_SURFACIC] = {1, 5, 2, 6, 2, 5, 2, 6, 3, 7, 3, 6, 3, 7, 4, 8, 4, 7, 5, 9, 6, 10, 6, 9, 6, 10, 7, 11, 7, 10, 7, 11, 8, 12, 8, 11, 9, 13, 10, 14, 10, 13, 10, 14, 11, 15, 11, 14, 11, 15, 12, 16, 12, 15, 49, 50, 53, 54, 53, 50, 50, 51, 54, 55, 54, 51, 51, 52, 55, 56, 55, 52, 53, 54, 57, 58, 57, 54, 54, 55, 58, 59, 58, 55, 55, 56, 59, 60, 59, 56, 57, 58, 61, 62, 61, 58, 58, 59, 62, 63, 62, 59, 59, 60, 63, 64, 63, 60, 5, 1, 17, 21, 9, 5, 21, 25, 13, 9, 25, 29, 21, 17, 33, 37, 25, 21, 37, 41, 29, 25, 41, 45, 37, 33, 49, 53, 41, 37, 53, 57, 45, 41, 57, 61, 4, 8, 24, 20, 8, 12, 28, 24, 12, 16, 32, 28, 20, 24, 40, 36, 24, 28, 44, 40, 28, 32, 48, 44, 36, 40, 56, 52, 40, 44, 60, 56, 44, 48, 64, 60, 17, 1, 2, 18, 33, 17, 18, 34, 49, 33, 34, 50, 18, 2, 3, 19, 34, 18, 19, 35, 50, 34, 35, 51, 19, 3, 4, 20, 35, 19, 20, 36, 51, 35, 36, 52, 13, 29, 30, 14, 29, 45, 46, 30, 45, 61, 62, 46, 14, 30, 31, 15, 30, 46, 47, 31, 46, 62, 63, 47, 15, 31, 32, 16, 31, 47, 48, 32, 47, 63, 64, 48, 18, 21, 5, 2, 22, 18, 2, 6, 19, 22, 6, 3, 23, 19, 3, 7, 23, 20, 4, 7, 21, 22, 6, 5, 21, 18, 17, 22, 23, 7, 6, 19, 22, 18, 18, 21, 22, 22, 25, 9, 6, 20, 23, 19, 24, 23, 7, 8, 26, 22, 6, 10, 22, 19, 23, 26, 23, 7, 10, 20, 23, 24, 22, 25, 21, 23, 27, 11, 7, 27, 24, 8, 11, 25, 26, 10, 9, 26, 23, 22, 22, 25, 26, 27, 24, 23, 27, 26, 10, 11, 23, 26, 27, 28, 27, 11, 12, 29, 26, 10, 13, 24, 27, 28, 26, 29, 25, 30, 26, 10, 14, 27, 30, 14, 11, 27, 30, 26, 31, 27, 11, 15, 26, 29, 30, 31, 28, 27, 28, 31, 15, 12, 27, 30, 31, 28, 31, 32, 37, 34, 33, 35, 38, 34, 37, 34, 38, 37, 34, 18, 21, 36, 39, 35, 35, 38, 39, 38, 34, 18, 22, 38, 35, 19, 22, 39, 36, 40, 35, 39, 23, 19, 41, 38, 37, 39, 36, 20, 23, 38, 37, 21, 22, 42, 39, 38, 38, 41, 42, 43, 40, 39, 39, 38, 22, 23, 39, 42, 43, 38, 41, 25, 22, 39, 40, 24, 23, 43, 40, 44, 45, 42, 41, 38, 42, 26, 22, 39, 42, 26, 23, 46, 43, 42, 39, 43, 27, 23, 42, 45, 46, 43, 40, 24, 27, 47, 44, 43, 41, 42, 26, 25, 43, 46, 47, 43, 42, 26, 27, 47, 44, 48, 42, 45, 29, 26, 43, 44, 28, 27, 46, 42, 26, 30, 46, 43, 27, 30, 47, 43, 27, 31, 44, 47, 31, 28, 53, 50, 34, 37, 54, 50, 34, 38, 51, 54, 38, 35, 51, 55, 39, 35, 54, 53, 37, 38, 55, 52, 36, 39, 55, 54, 38, 39, 56, 55, 39, 40, 57, 54, 38, 41, 58, 54, 38, 42, 55, 58, 42, 39, 55, 59, 43, 39, 58, 57, 41, 42, 59, 56, 40, 43, 59, 58, 42, 43, 58, 61, 45, 42, 59, 60, 44, 43, 62, 58, 42, 46, 59, 62, 46, 43, 59, 63, 47, 43, 60, 63, 47, 44};
      expected_parent_to_entity_idx[PDM_GEOMETRY_KIND_SURFACIC] = {0, 5, 10, 15, 20, 25, 30, 35, 40, 45, 50, 55, 60, 65, 70, 75, 80, 85, 90, 95, 100, 105, 110, 115, 120, 125, 130, 135, 140, 145, 150, 155, 160, 165, 170, 175, 180, 185, 190, 195, 200, 205, 210, 215, 220, 225, 230, 235, 240, 245, 250, 255, 260, 265, 270};
      expected_parent_to_entity    [PDM_GEOMETRY_KIND_SURFACIC] = {1, 37, 55, -73, -79, 2, 73, 74, 78, -82, 3, 58, -74, -75, 81, 4, 75, 76, 80, 87, 5, 61, -76, 77, 84, 6, 46, -77, -85, -89, 7, 38, -78, -83, 90, 8, 83, 86, 93, -95, 9, -80, -86, 88, -94, 10, -88, -91, -97, -98, 11, 85, 91, 92, -96, 12, 47, -92, -99, -101, 13, 39, -93, 100, 102, 14, 64, -100, 103, -107, 15, 97, -103, -104, 105, 16, 67, 104, 106, -110, 17, 99, -106, -108, -109, 18, 48, 70, 109, -111, 40, 56, 79, -112, 115, 82, 114, -115, 118, -124, 59, -81, 113, -118, 119, -87, -117, -119, -121, -128, 62, -84, 116, 121, 123, 49, 89, 120, -123, 131, 41, -90, -122, 124, -130, 95, -126, 130, -134, 141, 94, -125, 128, 134, -135, 98, -129, 135, -137, -143, 96, -127, -131, 137, 139, 50, 101, 132, -139, 146, 42, -102, -133, -141, -145, 65, 107, -138, 145, 147, -105, -136, 143, -147, 148, 68, 110, -142, -148, 149, 108, -140, -146, -149, -150, 51, 71, 111, 144, 150, 19, 43, 57, 112, 151, 20, -114, -151, 152, -155, 21, 60, -113, -152, -153, 22, 117, 153, -154, -157, 23, 63, -116, 154, 156, 24, 52, -120, -156, -158, 25, 44, 122, 155, 159, 26, 126, -159, 160, -163, 27, 125, 157, -160, -161, 28, 129, 161, -162, -165, 29, 127, 158, 162, 164, 30, 53, -132, -164, 167, 31, 45, 133, 163, -166, 32, 66, 138, 166, 168, 33, 136, 165, -168, -169, 34, 69, 142, 169, -170, 35, 140, -167, 170, -171, 36, 54, 72, -144, 171};
      expected_entity_to_vtx_idx   [PDM_GEOMETRY_KIND_RIDGE   ] = {0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56, 58, 60, 62, 64, 66, 68, 70, 72, 74, 76, 78, 80, 82, 84, 86, 88, 90, 92, 94, 96, 98, 100, 102, 104, 106, 108, 110, 112, 114, 116, 118, 120, 122, 124, 126, 128, 130, 132, 134, 136, 138, 140, 142, 144, 146, 148, 150, 152, 154, 156, 158, 160, 162, 164, 166, 168, 170, 172, 174, 176, 178, 180, 182, 184, 186, 188, 190, 192, 194, 196, 198, 200, 202, 204, 206, 208, 210, 212, 214, 216, 218, 220, 222, 224, 226, 228, 230, 232, 234, 236, 238, 240, 242, 244, 246, 248, 250, 252, 254, 256, 258, 260, 262, 264, 266, 268, 270, 272, 274, 276, 278, 280, 282, 284, 286, 288, 290, 292, 294, 296, 298, 300, 302, 304, 306, 308, 310, 312, 314, 316, 318, 320, 322, 324, 326, 328, 330, 332, 334, 336, 338, 340, 342, 344, 346, 348, 350, 352, 354, 356, 358, 360};
      expected_entity_to_vtx       [PDM_GEOMETRY_KIND_RIDGE   ] = {1, 2, 2, 3, 3, 4, 14, 13, 15, 14, 16, 15, 5, 1, 9, 5, 13, 9, 4, 8, 8, 12, 12, 16, 49, 50, 50, 51, 51, 52, 61, 62, 62, 63, 63, 64, 49, 53, 53, 57, 57, 61, 52, 56, 56, 60, 60, 64, 1, 17, 17, 33, 33, 49, 4, 20, 20, 36, 36, 52, 13, 29, 29, 45, 45, 61, 16, 32, 32, 48, 48, 64, 2, 5, 6, 2, 3, 6, 7, 3, 6, 5, 7, 4, 7, 6, 7, 8, 9, 6, 10, 6, 10, 7, 7, 11, 11, 8, 10, 9, 2, 18, 10, 11, 3, 19, 10, 13, 11, 12, 14, 10, 11, 14, 5, 21, 11, 15, 12, 15, 6, 22, 7, 23, 8, 24, 9, 25, 17, 18, 10, 26, 18, 19, 11, 27, 21, 17, 21, 18, 19, 20, 12, 28, 22, 18, 22, 19, 19, 23, 23, 20, 21, 22, 20, 24, 14, 30, 22, 23, 15, 31, 25, 21, 24, 23, 25, 22, 26, 22, 23, 26, 23, 27, 27, 24, 25, 26, 24, 28, 18, 34, 27, 26, 19, 35, 29, 25, 28, 27, 26, 29, 30, 26, 27, 30, 21, 37, 31, 27, 31, 28, 30, 29, 28, 32, 22, 38, 31, 30, 23, 39, 32, 31, 24, 40, 25, 41, 33, 34, 26, 42, 34, 35, 27, 43, 37, 33, 37, 34, 35, 36, 38, 34, 28, 44, 38, 35, 35, 39, 38, 37, 36, 39, 30, 46, 36, 40, 39, 38, 31, 47, 41, 37, 39, 40, 41, 38, 42, 38, 39, 42, 43, 39, 42, 41, 43, 40, 40, 44, 34, 50, 42, 43, 35, 51, 45, 41, 42, 45, 44, 43, 46, 42, 43, 46, 47, 43, 37, 53, 47, 44, 46, 45, 38, 54, 44, 48, 47, 46, 39, 55, 48, 47, 40, 56, 41, 57, 42, 58, 43, 59, 53, 50, 54, 50, 44, 60, 51, 54, 55, 51, 52, 55, 53, 54, 46, 62, 54, 55, 47, 63, 55, 56, 54, 57, 54, 58, 58, 55, 55, 59, 56, 59, 57, 58, 59, 58, 61, 58, 59, 60, 62, 58, 62, 59, 59, 63, 63, 60};
      expected_parent_to_entity_idx[PDM_GEOMETRY_KIND_RIDGE   ] = {0, 9, 18, 27, 36, 45, 54, 63, 72, 81, 90, 99, 108, 117, 126, 135, 144, 153, 162, 171, 180, 189, 198, 207, 216, 225, 234, 243, 252, 261, 270, 279, 288, 297, 306, 315, 324, 333, 342, 351, 360, 369, 378, 387, 396, 405, 414, 423, 432, 441, 450, 459, 468, 477, 486};
      expected_parent_to_entity    [PDM_GEOMETRY_KIND_RIDGE   ] = {1, 7, 25, 37, 51, 58, 65, 69, -70, -37, -38, 41, 51, 58, 61, 70, -73, -77, 2, 38, 39, 51, 53, 61, 67, 73, -74, -39, -40, 43, 53, 61, 62, 74, 75, -80, 3, 28, 40, -42, 53, 62, 71, -75, -76, 10, 28, 42, -44, 62, 63, 76, 78, 83, 8, -41, -45, 58, 61, 64, 77, 82, -84, 45, -46, 50, 61, 64, 66, 84, -85, -89, -43, 46, -47, 61, 62, 66, 80, 85, 86, 47, 48, -52, 62, 66, 68, -86, 87, 92, 44, -48, -49, 62, 63, 68, -83, -87, -88, 11, 49, -55, 63, 68, 72, 88, 90, 95, 9, 31, -50, 54, 64, 66, 89, 94, 96, 4, 31, -54, -56, 66, 79, -96, -97, 102, 52, 56, 57, 66, 68, 79, -92, 97, 98, 5, -57, 59, 68, 79, 81, -98, -100, 105, 55, -59, 60, 68, 72, 81, -95, 100, -101, 6, 12, 34, -60, 72, 81, 101, 103, 107, 26, 65, 69, -70, 91, 99, 110, 114, -115, 70, -73, -77, 91, 99, 104, 115, -117, 121, 67, 73, -74, 91, 93, 104, 112, 117, -119, 74, 75, -80, 93, 104, 106, 119, 120, 125, 29, 71, -75, -76, 93, 106, 116, -120, 122, 29, 76, 78, 83, 106, 108, -122, 124, -128, 77, 82, -84, 99, 104, 109, -121, 127, -129, 84, -85, -89, 104, 109, 111, 129, -130, 133, 80, 85, 86, 104, 106, 111, -125, 130, 131, -86, 87, 92, 106, 111, 113, -131, -132, -137, -83, -87, -88, 106, 108, 113, 128, 132, -134, 88, 90, 95, 108, 113, 118, 134, 135, 141, 32, 89, 94, 96, 109, 111, -133, 139, 140, 32, -96, -97, 102, 111, 123, -140, -142, 147, -92, 97, 98, 111, 113, 123, 137, 142, 143, -98, -100, 105, 113, 123, 126, -143, -144, 150, -95, 100, -101, 113, 118, 126, -141, 144, -146, 35, 101, 103, 107, 118, 126, 146, 149, 152, 13, -19, 27, 110, 114, -115, 136, 145, -157, 115, -117, 121, 136, 145, 148, 157, -158, -163, 14, 112, 117, -119, 136, 138, 148, 158, 160, 119, 120, 125, 138, 148, 151, -160, -161, -165, 15, 30, 116, -120, 122, 138, 151, 161, 162, 22, 30, -122, 124, -128, 151, 153, -162, -167, -20, -121, 127, -129, 145, 148, 154, 163, 168, 129, -130, 133, 148, 154, 155, -168, 169, -173, -125, 130, 131, 148, 151, 155, 165, -169, -170, -131, -132, -137, 151, 155, 156, 170, 171, 174, 128, 132, -134, 151, 153, 156, 167, -171, 172, 23, 134, 135, 141, 153, 156, 159, -172, -176, -21, 33, -133, 139, 140, 154, 155, 173, -175, -16, 33, -140, -142, 147, 155, 164, 175, -177, 137, 142, 143, 155, 156, 164, -174, 177, -178, -17, -143, -144, 150, 156, 164, 166, 178, 179, -141, 144, -146, 156, 159, 166, 176, -179, -180, -18, 24, 36, 146, 149, 152, 159, 166, 180};
    };
    SUBCASE("PRISM6, compute_parent_child = 0") {
      printf("PRISM6, compute_parent_child = 0\n");
      compute_parent_child = PDM_FALSE;
    };
  }

  SUBCASE("HEXA8") {
    elt_type = PDM_MESH_NODAL_HEXA8;
    expected_child_to_parent_idx[PDM_GEOMETRY_KIND_SURFACIC] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 52, 53, 54};
    expected_child_to_parent    [PDM_GEOMETRY_KIND_SURFACIC] = {1, 2, 3, 4, 5, 6, 7, 8, 9, 19, 20, 21, 22, 23, 24, 25, 26, 27, 1, 4, 7, 10, 13, 16, 19, 22, 25, 3, 6, 9, 12, 15, 18, 21, 24, 27, 1, 10, 19, 2, 11, 20, 3, 12, 21, 7, 16, 25, 8, 17, 26, 9, 18, 27};
    expected_child_to_parent_idx[PDM_GEOMETRY_KIND_RIDGE   ] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36};
    expected_child_to_parent    [PDM_GEOMETRY_KIND_RIDGE   ] = {1, 2, 3, 7, 8, 9, 1, 4, 7, 3, 6, 9, 19, 20, 21, 25, 26, 27, 19, 22, 25, 21, 24, 27, 1, 10, 19, 3, 12, 21, 7, 16, 25, 9, 18, 27};
    SUBCASE("HEXA8, compute_parent_child = 1") {
      printf("HEXA8, compute_parent_child = 1\n");
      compute_parent_child = PDM_TRUE;
      expected_n_entity[PDM_GEOMETRY_KIND_SURFACIC] = 108;
      expected_n_entity[PDM_GEOMETRY_KIND_RIDGE   ] = 144;
      expected_entity_to_vtx_idx   [PDM_GEOMETRY_KIND_SURFACIC] = {0, 4, 8, 12, 16, 20, 24, 28, 32, 36, 40, 44, 48, 52, 56, 60, 64, 68, 72, 76, 80, 84, 88, 92, 96, 100, 104, 108, 112, 116, 120, 124, 128, 132, 136, 140, 144, 148, 152, 156, 160, 164, 168, 172, 176, 180, 184, 188, 192, 196, 200, 204, 208, 212, 216, 220, 224, 228, 232, 236, 240, 244, 248, 252, 256, 260, 264, 268, 272, 276, 280, 284, 288, 292, 296, 300, 304, 308, 312, 316, 320, 324, 328, 332, 336, 340, 344, 348, 352, 356, 360, 364, 368, 372, 376, 380, 384, 388, 392, 396, 400, 404, 408, 412, 416, 420, 424, 428, 432};
      expected_entity_to_vtx       [PDM_GEOMETRY_KIND_SURFACIC] = {2, 1, 5, 6, 3, 2, 6, 7, 4, 3, 7, 8, 6, 5, 9, 10, 7, 6, 10, 11, 8, 7, 11, 12, 10, 9, 13, 14, 11, 10, 14, 15, 12, 11, 15, 16, 49, 50, 54, 53, 50, 51, 55, 54, 51, 52, 56, 55, 53, 54, 58, 57, 54, 55, 59, 58, 55, 56, 60, 59, 57, 58, 62, 61, 58, 59, 63, 62, 59, 60, 64, 63, 5, 1, 17, 21, 9, 5, 21, 25, 13, 9, 25, 29, 21, 17, 33, 37, 25, 21, 37, 41, 29, 25, 41, 45, 37, 33, 49, 53, 41, 37, 53, 57, 45, 41, 57, 61, 4, 8, 24, 20, 8, 12, 28, 24, 12, 16, 32, 28, 20, 24, 40, 36, 24, 28, 44, 40, 28, 32, 48, 44, 36, 40, 56, 52, 40, 44, 60, 56, 44, 48, 64, 60, 17, 1, 2, 18, 33, 17, 18, 34, 49, 33, 34, 50, 18, 2, 3, 19, 34, 18, 19, 35, 50, 34, 35, 51, 19, 3, 4, 20, 35, 19, 20, 36, 51, 35, 36, 52, 13, 29, 30, 14, 29, 45, 46, 30, 45, 61, 62, 46, 14, 30, 31, 15, 30, 46, 47, 31, 46, 62, 63, 47, 15, 31, 32, 16, 31, 47, 48, 32, 47, 63, 64, 48, 6, 22, 18, 2, 19, 23, 7, 3, 6, 22, 21, 5, 7, 23, 22, 6, 23, 24, 8, 7, 10, 26, 22, 6, 11, 27, 23, 7, 10, 26, 25, 9, 26, 27, 11, 10, 27, 28, 12, 11, 21, 22, 18, 17, 14, 30, 26, 10, 23, 22, 18, 19, 27, 31, 15, 11, 23, 24, 20, 19, 26, 25, 21, 22, 26, 27, 23, 22, 28, 27, 23, 24, 29, 30, 26, 25, 34, 38, 22, 18, 30, 31, 27, 26, 23, 39, 35, 19, 31, 32, 28, 27, 37, 38, 22, 21, 38, 39, 23, 22, 24, 40, 39, 23, 38, 42, 26, 22, 39, 43, 27, 23, 41, 42, 26, 25, 42, 43, 27, 26, 43, 44, 28, 27, 38, 37, 33, 34, 42, 46, 30, 26, 39, 38, 34, 35, 31, 47, 43, 27, 39, 40, 36, 35, 41, 42, 38, 37, 42, 43, 39, 38, 43, 44, 40, 39, 46, 45, 41, 42, 38, 54, 50, 34, 46, 47, 43, 42, 51, 55, 39, 35, 48, 47, 43, 44, 53, 54, 38, 37, 39, 55, 54, 38, 55, 56, 40, 39, 54, 58, 42, 38, 43, 59, 55, 39, 57, 58, 42, 41, 43, 59, 58, 42, 59, 60, 44, 43, 58, 62, 46, 42, 47, 63, 59, 43};
      expected_parent_to_entity_idx[PDM_GEOMETRY_KIND_SURFACIC] = {0, 6, 12, 18, 24, 30, 36, 42, 48, 54, 60, 66, 72, 78, 84, 90, 96, 102, 108, 114, 120, 126, 132, 138, 144, 150, 156, 162};
      expected_parent_to_entity    [PDM_GEOMETRY_KIND_SURFACIC] = {1, 19, 37, 55, -57, -65, 2, 40, -55, -56, -58, 67, 3, 28, 43, 56, 59, -69, 4, 20, 57, 60, -62, 70, 5, 58, -60, 61, 63, -71, 6, 29, -59, -61, 64, 72, 7, 21, 46, 62, 66, -73, 8, 49, -63, -66, -68, -75, 9, 30, 52, -64, 68, -77, 22, 38, 65, -74, 78, 86, 41, -67, 74, 76, 79, 88, 31, 44, 69, -76, -80, -90, 23, -70, -78, -81, 83, -91, 71, -79, 81, -82, 84, -92, 32, -72, 80, 82, 85, -93, 24, 47, 73, -83, -87, 94, 50, 75, -84, 87, 89, -96, 33, 53, 77, -85, -89, 98, 10, 25, 39, -86, 95, 99, 11, 42, -88, -95, -97, -100, 12, 34, 45, 90, 97, 101, 13, 26, 91, -99, -102, 104, 14, 92, 100, 102, 103, -105, 15, 35, 93, -101, -103, 106, 16, 27, 48, -94, -104, -107, 17, 51, 96, 105, 107, 108, 18, 36, 54, -98, -106, -108};
      expected_entity_to_vtx_idx   [PDM_GEOMETRY_KIND_RIDGE   ] = {0, 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56, 58, 60, 62, 64, 66, 68, 70, 72, 74, 76, 78, 80, 82, 84, 86, 88, 90, 92, 94, 96, 98, 100, 102, 104, 106, 108, 110, 112, 114, 116, 118, 120, 122, 124, 126, 128, 130, 132, 134, 136, 138, 140, 142, 144, 146, 148, 150, 152, 154, 156, 158, 160, 162, 164, 166, 168, 170, 172, 174, 176, 178, 180, 182, 184, 186, 188, 190, 192, 194, 196, 198, 200, 202, 204, 206, 208, 210, 212, 214, 216, 218, 220, 222, 224, 226, 228, 230, 232, 234, 236, 238, 240, 242, 244, 246, 248, 250, 252, 254, 256, 258, 260, 262, 264, 266, 268, 270, 272, 274, 276, 278, 280, 282, 284, 286, 288};
      expected_entity_to_vtx       [PDM_GEOMETRY_KIND_RIDGE   ] = {1, 2, 2, 3, 3, 4, 14, 13, 15, 14, 16, 15, 5, 1, 9, 5, 13, 9, 4, 8, 8, 12, 12, 16, 49, 50, 50, 51, 51, 52, 61, 62, 62, 63, 63, 64, 49, 53, 53, 57, 57, 61, 52, 56, 56, 60, 60, 64, 1, 17, 17, 33, 33, 49, 4, 20, 20, 36, 36, 52, 13, 29, 29, 45, 45, 61, 16, 32, 32, 48, 48, 64, 6, 2, 3, 7, 5, 6, 6, 7, 7, 8, 10, 6, 11, 7, 10, 9, 2, 18, 11, 10, 3, 19, 11, 12, 10, 14, 15, 11, 5, 21, 6, 22, 7, 23, 8, 24, 9, 25, 17, 18, 10, 26, 18, 19, 21, 17, 11, 27, 19, 20, 22, 18, 12, 28, 19, 23, 21, 22, 14, 30, 20, 24, 22, 23, 25, 21, 15, 31, 24, 23, 26, 22, 27, 23, 25, 26, 24, 28, 18, 34, 27, 26, 19, 35, 29, 25, 28, 27, 26, 30, 21, 37, 27, 31, 30, 29, 22, 38, 28, 32, 31, 30, 23, 39, 32, 31, 24, 40, 25, 41, 33, 34, 26, 42, 34, 35, 27, 43, 37, 33, 35, 36, 34, 38, 28, 44, 39, 35, 37, 38, 36, 40, 30, 46, 38, 39, 31, 47, 41, 37, 40, 39, 38, 42, 43, 39, 41, 42, 34, 50, 40, 44, 43, 42, 35, 51, 45, 41, 43, 44, 42, 46, 37, 53, 43, 47, 46, 45, 44, 48, 38, 54, 47, 46, 39, 55, 48, 47, 40, 56, 41, 57, 42, 58, 43, 59, 44, 60, 54, 50, 55, 51, 54, 53, 46, 62, 54, 55, 47, 63, 55, 56, 58, 54, 55, 59, 58, 57, 59, 58, 59, 60, 62, 58, 63, 59};
      expected_parent_to_entity_idx[PDM_GEOMETRY_KIND_RIDGE   ] = {0, 12, 24, 36, 48, 60, 72, 84, 96, 108, 120, 132, 144, 156, 168, 180, 192, 204, 216, 228, 240, 252, 264, 276, 288, 300, 312, 324};
      expected_parent_to_entity    [PDM_GEOMETRY_KIND_RIDGE   ] = {1, 7, 25, -37, -39, 45, 51, 52, 56, 59, -62, -65, 2, 37, 38, -40, 45, 47, 52, 53, 58, 62, 64, -68, 3, 10, 28, -38, -41, 47, 53, 54, 61, -64, 67, 71, 8, 39, -42, 44, 51, 52, 55, 57, 65, 69, -72, -74, 40, 42, -43, 46, 52, 53, 57, 60, 68, 72, -73, 77, 11, 41, 43, -48, 53, 54, 60, 63, -71, 73, 75, 80, 4, 9, 31, -44, 49, 55, 57, 66, 74, 79, 81, 84, 5, -46, -49, -50, 57, 60, 66, 70, -77, -81, 83, 87, 6, 12, 34, 48, 50, 60, 63, 70, -80, -83, 86, 89, 26, 56, 59, -62, -65, 76, 82, 85, 92, 96, 98, -101, 58, 62, 64, -68, 76, 78, 85, 88, 94, -98, -100, -104, 29, 61, -64, 67, 71, 78, 88, 90, 97, 100, 102, 107, 65, 69, -72, -74, 82, 85, 91, 93, 101, 106, 108, -110, 68, 72, -73, 77, 85, 88, 93, 95, 104, -108, -109, 113, -71, 73, 75, 80, 88, 90, 95, 99, -107, 109, 112, -116, 32, 74, 79, 81, 84, 91, 93, 103, 110, 115, 117, 120, -77, -81, 83, 87, 93, 95, 103, 105, -113, -117, 119, 123, 35, -80, -83, 86, 89, 95, 99, 105, 116, -119, 121, 125, 13, -19, 27, 92, 96, 98, -101, 111, 118, 122, -131, 133, 14, 94, -98, -100, -104, 111, 114, 122, 124, 131, -132, -135, 15, 22, 30, 97, 100, 102, 107, 114, 124, 126, 132, -137, -20, 101, 106, 108, -110, 118, 122, 127, 128, -133, -138, 140, 104, -108, -109, 113, 122, 124, 128, 129, 135, 138, 139, 141, 23, -107, 109, 112, -116, 124, 126, 129, 130, 137, -139, -142, -16, -21, 33, 110, 115, 117, 120, 127, 128, 134, -140, -143, -17, -113, -117, 119, 123, 128, 129, 134, 136, -141, 143, -144, -18, 24, 36, 116, -119, 121, 125, 129, 130, 136, 142, 144};
    };
    SUBCASE("HEXA8, compute_parent_child = 0") {
      printf("HEXA8, compute_parent_child = 0\n");
      compute_parent_child = PDM_FALSE;
    };
  }



  /* Generate mesh */
  PDM_part_mesh_nodal_t *pmn = _generate_mesh(pdm_comm, elt_type);


  /* Compute link between child and parent entities */
  PDM_geometry_kind_t geom_kind_parent = PDM_GEOMETRY_KIND_MAX;
  int mesh_dimension = PDM_Mesh_nodal_elt_dim_get(elt_type);
  if (mesh_dimension == 3) {
    geom_kind_parent = PDM_GEOMETRY_KIND_VOLUMIC;
  }
  else if (mesh_dimension == 2) {
    geom_kind_parent = PDM_GEOMETRY_KIND_SURFACIC;
  }
  else {
    PDM_error(__FILE__, __LINE__, 0, "1D not yet available\n");
  }

  PDM_part_mesh_nodal_elmts_t *pmne_parent = PDM_part_mesh_nodal_part_mesh_nodal_elmts_get(pmn,
                                                                                           geom_kind_parent);

  int n_parent = PDM_part_mesh_nodal_elmts_n_elmts_get(pmne_parent, 0);

  int *pn_vtx = NULL;
  PDM_malloc(pn_vtx, 1, int);
  pn_vtx[0] = PDM_part_mesh_nodal_n_vtx_get(pmn, 0);


  for (int geom_kind = (int) geom_kind_parent + 1; geom_kind < (int) PDM_GEOMETRY_KIND_CORNER; geom_kind++) {

    PDM_part_mesh_nodal_elmts_t *pmne_child = PDM_part_mesh_nodal_part_mesh_nodal_elmts_get(pmn,
                                                                                            (PDM_geometry_kind_t) geom_kind);

    int **child_to_parent_idx  = NULL;
    int **child_to_parent      = NULL;
    int  *n_entity             = NULL;
    int **entity_to_vtx_idx    = NULL;
    int **entity_to_vtx        = NULL;
    int **parent_to_entity_idx = NULL;
    int **parent_to_entity     = NULL;

    PDM_part_mesh_nodal_elmts_compute_child_parent(pmne_parent,
                                                   pmne_child,
                                                   PDM_MESH_ENTITY_MAX, // unused here
                                                   compute_parent_child,
                                                   pn_vtx,
                                                   &child_to_parent_idx,
                                                   &child_to_parent,
                                                   &n_entity,
                                                   &entity_to_vtx_idx,
                                                   &entity_to_vtx,
                                                   &parent_to_entity_idx,
                                                   &parent_to_entity);

    int n_child = PDM_part_mesh_nodal_elmts_n_elmts_get(pmne_child, 0);

    /* Check */

    // Child -> parent
    MPI_CHECK_EQ_C_ARRAY(0, child_to_parent_idx[0], expected_child_to_parent_idx[geom_kind].data(), n_child+1);
    MPI_CHECK_EQ_C_ARRAY(0, child_to_parent    [0], expected_child_to_parent    [geom_kind].data(), child_to_parent_idx[0][n_child]);


    if (compute_parent_child) {
      // Entity -> vtx
      CHECK(n_entity[0] == expected_n_entity[geom_kind]);
      MPI_CHECK_EQ_C_ARRAY(0, entity_to_vtx_idx[0], expected_entity_to_vtx_idx[geom_kind].data(), n_child+1);
      MPI_CHECK_EQ_C_ARRAY(0, entity_to_vtx    [0], expected_entity_to_vtx    [geom_kind].data(), child_to_parent_idx[0][n_child]);

      // Parent -> child
      MPI_CHECK_EQ_C_ARRAY(0, parent_to_entity_idx[0], expected_parent_to_entity_idx[geom_kind].data(), n_parent+1);
      MPI_CHECK_EQ_C_ARRAY(0, parent_to_entity    [0], expected_parent_to_entity    [geom_kind].data(), parent_to_entity_idx[0][n_parent]);
    }

    /* Free memory */
    PDM_free(pn_vtx);
    PDM_free(child_to_parent_idx[0]);
    PDM_free(child_to_parent    [0]);
    if (compute_parent_child) {
      PDM_free(entity_to_vtx_idx   [0]);
      PDM_free(entity_to_vtx       [0]);
      PDM_free(parent_to_entity_idx[0]);
      PDM_free(parent_to_entity    [0]);
    }

    PDM_free(child_to_parent_idx );
    PDM_free(child_to_parent     );
    PDM_free(n_entity            );
    PDM_free(entity_to_vtx_idx   );
    PDM_free(entity_to_vtx       );
    PDM_free(parent_to_entity_idx);
    PDM_free(parent_to_entity    );
  }

  PDM_part_mesh_nodal_free(pmn);
}

